@@ -1,5 +1,4 @@
 # Copyright (c) 2006-2015 LOGILAB S.A. (Paris, FRANCE) <contact@logilab.fr>
-# Copyright (c) 2010 Julien Jehannet <julien.jehannet@logilab.fr>
 # Copyright (c) 2013-2014 Google, Inc.
 # Copyright (c) 2014-2020 Claudiu Popa <pcmanticore@gmail.com>
 # Copyright (c) 2014 Brett Cannon <brett@python.org>
@@ -32,7 +31,7 @@
 from os.path import join
 
 # For an official release, use dev_version = None
-numversion = (2, 7, 1)
+numversion = (2, 7, 2)
 dev_version = None
 
 version = ".".join(str(num) for num in numversion)
@@ -40,7 +39,7 @@ if dev_version is not None:
     version += "-dev" + str(dev_version)
 
 install_requires = [
-    "astroid==2.5.0",
+    "astroid>=2.5.1,<2.6",
     "isort>=4.2.5,<6",
     "mccabe>=0.6,<0.7",
     "toml>=0.7.1",
@@ -50,9 +49,9 @@ dependency_links = []  # type: ignore
 
 extras_require = {}
 extras_require[':sys_platform=="win32"'] = ["colorama"]
-extras_require["docs"] = ["sphinx~=3.2", "python-docs-theme"]
+extras_require["docs"] = ["sphinx==3.5.1", "python-docs-theme==2020.12"]
 
-license = "GPL"
+license = "GPL-2.0-or-later"
 description = "python code static checker"
 web = "https://github.com/PyCQA/pylint"
 mailinglist = "mailto:code-quality@python.org"
