@@ -1,9 +1,8 @@
-# -*- coding: utf-8 -*-
 # Copyright (c) 2006-2016 LOGILAB S.A. (Paris, FRANCE) <contact@logilab.fr>
 # Copyright (c) 2010 Maarten ter Huurne <maarten@treewalker.org>
 # Copyright (c) 2012-2014 Google, Inc.
 # Copyright (c) 2012 FELD Boris <lothiraldan@gmail.com>
-# Copyright (c) 2013-2018 Claudiu Popa <pcmanticore@gmail.com>
+# Copyright (c) 2013-2020 Claudiu Popa <pcmanticore@gmail.com>
 # Copyright (c) 2014 Michal Nowikowski <godfryd@gmail.com>
 # Copyright (c) 2014 Brett Cannon <brett@python.org>
 # Copyright (c) 2014 Arun Persaud <arun@nubati.net>
@@ -16,13 +15,25 @@
 # Copyright (c) 2016 Florian Bruhin <me@the-compiler.org>
 # Copyright (c) 2016 Moises Lopez <moylop260@vauxoo.com>
 # Copyright (c) 2016 Jakub Wilk <jwilk@jwilk.net>
-# Copyright (c) 2017 hippo91 <guillaume.peillex@gmail.com>
+# Copyright (c) 2017, 2019-2020 hippo91 <guillaume.peillex@gmail.com>
+# Copyright (c) 2018, 2020 Anthony Sottile <asottile@umich.edu>
+# Copyright (c) 2018-2019 Nick Drozd <nicholasdrozd@gmail.com>
+# Copyright (c) 2018-2019 Ashley Whetter <ashley@awhetter.co.uk>
+# Copyright (c) 2018 Lucas Cimon <lucas.cimon@gmail.com>
+# Copyright (c) 2018 Bryce Guinta <bryce.paul.guinta@gmail.com>
 # Copyright (c) 2018 ssolanki <sushobhitsolanki@gmail.com>
-# Copyright (c) 2018 Ashley Whetter <ashley@awhetter.co.uk>
-# Copyright (c) 2018 Anthony Sottile <asottile@umich.edu>
 # Copyright (c) 2018 Ben Green <benhgreen@icloud.com>
-# Copyright (c) 2018 Ville Skyttä <ville.skytta@upcloud.com>
-# Copyright (c) 2018 Nick Drozd <nicholasdrozd@gmail.com>
+# Copyright (c) 2018 Ville Skyttä <ville.skytta@iki.fi>
+# Copyright (c) 2019-2021 Pierre Sassoulas <pierre.sassoulas@gmail.com>
+# Copyright (c) 2019 mattlbeck <17108752+mattlbeck@users.noreply.github.com>
+# Copyright (c) 2019-2020 craig-sh <craig-sh@users.noreply.github.com>
+# Copyright (c) 2019 Janne Rönkkö <jannero@users.noreply.github.com>
+# Copyright (c) 2019 Hugo van Kemenade <hugovk@users.noreply.github.com>
+# Copyright (c) 2019 Grygorii Iermolenko <gyermolenko@gmail.com>
+# Copyright (c) 2019 Andrzej Klajnert <github@aklajnert.pl>
+# Copyright (c) 2019 Pascal Corpet <pcorpet@users.noreply.github.com>
+# Copyright (c) 2020 GergelyKalmar <gergely.kalmar@logikal.jp>
+# Copyright (c) 2021 Marc Mueller <30130371+cdce8p@users.noreply.github.com>
 
 # Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
 # For details: https://github.com/PyCQA/pylint/blob/master/COPYING
@@ -52,6 +63,7 @@ from pylint.checkers.utils import (
     is_builtin_object,
     is_comprehension,
     is_iterable,
+    is_overload_stub,
     is_property_setter,
     is_property_setter_or_deleter,
     is_protocol_class,
@@ -110,7 +122,11 @@ def _signature_from_call(call):
 def _signature_from_arguments(arguments):
     kwarg = arguments.kwarg
     vararg = arguments.vararg
-    args = [arg.name for arg in arguments.args if arg.name != "self"]
+    args = [
+        arg.name
+        for arg in chain(arguments.posonlyargs, arguments.args)
+        if arg.name != "self"
+    ]
     kwonlyargs = [arg.name for arg in arguments.kwonlyargs]
     return _ParameterSignature(args, kwonlyargs, vararg, kwarg)
 
@@ -231,15 +247,16 @@ def _has_different_parameters_default_value(original, overridden):
             astroid.ClassDef: "name",
             astroid.Tuple: "elts",
             astroid.List: "elts",
+            astroid.Dict: "items",
         }
         handled_types = tuple(
             astroid_type for astroid_type in astroid_type_compared_attr
         )
         original_type = _get_node_type(original_default, handled_types)
         if original_type:
-            #  We handle only astroid types that are inside the dict astroid_type_compared_attr
+            # We handle only astroid types that are inside the dict astroid_type_compared_attr
             if not isinstance(overridden_default, original_type):
-                #  Two args with same name but different types
+                # Two args with same name but different types
                 return True
             if not _check_arg_equality(
                 original_default,
@@ -281,11 +298,30 @@ def _different_parameters(original, overridden, dummy_parameter_regex):
     original_parameters = _positional_parameters(original)
     overridden_parameters = _positional_parameters(overridden)
 
+    # Copy kwonlyargs list so that we don't affect later function linting
+    original_kwonlyargs = original.args.kwonlyargs
+
+    # Allow positional/keyword variadic in overridden to match against any
+    # positional/keyword argument in original.
+    # Keep any arguments that are found seperately in overridden to satisfy
+    # later tests
+    if overridden.args.vararg:
+        overidden_names = [v.name for v in overridden_parameters]
+        original_parameters = [
+            v for v in original_parameters if v.name in overidden_names
+        ]
+
+    if overridden.args.kwarg:
+        overidden_names = [v.name for v in overridden.args.kwonlyargs]
+        original_kwonlyargs = [
+            v for v in original.args.kwonlyargs if v.name in overidden_names
+        ]
+
     different_positional = _has_different_parameters(
         original_parameters, overridden_parameters, dummy_parameter_regex
     )
     different_kwonly = _has_different_parameters(
-        original.args.kwonlyargs, overridden.args.kwonlyargs, dummy_parameter_regex
+        original_kwonlyargs, overridden.args.kwonlyargs, dummy_parameter_regex
     )
     if original.name in PYMETHODS:
         # Ignore the difference for special methods. If the parameter
@@ -295,21 +331,12 @@ def _different_parameters(original, overridden, dummy_parameter_regex):
         # be used as keyword arguments anyway.
         different_positional = different_kwonly = False
 
-    # Both or none should have extra variadics, otherwise the method
-    # loses or gains capabilities that are not reflected into the parent method,
-    # leading to potential inconsistencies in the code.
-    different_kwarg = (
-        sum(1 for param in (original.args.kwarg, overridden.args.kwarg) if not param)
-        == 1
-    )
-    different_vararg = (
-        sum(1 for param in (original.args.vararg, overridden.args.vararg) if not param)
-        == 1
-    )
+    # Arguments will only violate LSP if there are variadics in the original
+    # that are then removed from the overridden
+    kwarg_lost = original.args.kwarg and not overridden.args.kwarg
+    vararg_lost = original.args.vararg and not overridden.args.vararg
 
-    return any(
-        (different_positional, different_kwarg, different_vararg, different_kwonly)
-    )
+    return any((different_positional, kwarg_lost, vararg_lost, different_kwonly))
 
 
 def _is_invalid_base_class(cls):
@@ -336,7 +363,7 @@ def _has_data_descriptor(cls, attr):
 
 
 def _called_in_methods(func, klass, methods):
-    """ Check if the func was called in any of the given methods,
+    """Check if the func was called in any of the given methods,
     belonging to the *klass*. Returns True if so, False otherwise.
     """
     if not isinstance(func, astroid.FunctionDef):
@@ -363,8 +390,7 @@ def _called_in_methods(func, klass, methods):
 
 
 def _is_attribute_property(name, klass):
-    """ Check if the given attribute *name* is a property
-    in the given *klass*.
+    """Check if the given attribute *name* is a property in the given *klass*.
 
     It will look for `property` calls or for functions
     with the given name, decorated by `property` or `property`
@@ -377,7 +403,7 @@ def _is_attribute_property(name, klass):
         attributes = klass.getattr(name)
     except astroid.NotFoundError:
         return False
-    property_name = "{}.property".format(BUILTINS)
+    property_name = f"{BUILTINS}.property"
     for attr in attributes:
         if attr is astroid.Uninferable:
             continue
@@ -389,8 +415,13 @@ def _is_attribute_property(name, klass):
             inferred
         ):
             return True
-        if inferred.pytype() == property_name:
-            return True
+        if inferred.pytype() != property_name:
+            continue
+
+        cls = node_frame_class(inferred)
+        if cls == klass.declared_metaclass():
+            continue
+        return True
     return False
 
 
@@ -423,6 +454,7 @@ def _safe_infer_call_result(node, caller, context=None):
         return None  # there is some kind of ambiguity
     except StopIteration:
         return value
+    return None
 
 
 def _has_same_layout_slots(slots, assigned_value):
@@ -564,9 +596,9 @@ MSGS = {
     "W0236": (
         "Method %r was expected to be %r, found it instead as %r",
         "invalid-overridden-method",
-        "Used when we detect that a method was overridden as a property "
-        "or the other way around, which could result in potential bugs at "
-        "runtime.",
+        "Used when we detect that a method was overridden in a way "
+        "that does not match its base class "
+        "which could result in potential bugs at runtime.",
     ),
     "E0236": (
         "Invalid object %r in __slots__, must contain only non empty strings",
@@ -634,11 +666,15 @@ MSGS = {
 }
 
 
+def _scope_default():
+    return collections.defaultdict(list)
+
+
 class ScopeAccessMap:
     """Store the accessed variables per scope."""
 
     def __init__(self):
-        self._scopes = collections.defaultdict(lambda: collections.defaultdict(list))
+        self._scopes = collections.defaultdict(_scope_default)
 
     def set_accessed(self, node):
         """Set the given node as accessed."""
@@ -721,6 +757,15 @@ a metaclass class method.",
                 ),
             },
         ),
+        (
+            "check-protected-access-in-special-methods",
+            {
+                "default": False,
+                "type": "yn",
+                "metavar": "<y or n>",
+                "help": "Warn about protected attribute access inside special methods",
+            },
+        ),
     )
 
     def __init__(self, linter=None):
@@ -750,8 +795,7 @@ a metaclass class method.",
         "duplicate-bases",
     )
     def visit_classdef(self, node):
-        """init visit variable _accessed
-        """
+        """init visit variable _accessed"""
         self._check_bases_classes(node)
         # if not an exception or a metaclass
         if node.type == "class" and has_known_bases(node):
@@ -782,10 +826,10 @@ a metaclass class method.",
         """
         for base in node.bases:
             ancestor = safe_infer(base)
-            if ancestor in (astroid.Uninferable, None):
+            if not ancestor:
                 continue
             if isinstance(ancestor, astroid.Instance) and ancestor.is_subtype_of(
-                "%s.type" % (BUILTINS,)
+                f"{BUILTINS}.type"
             ):
                 continue
 
@@ -948,11 +992,19 @@ a metaclass class method.",
                 and overridden_frame.type == "method"
             ):
                 overridden_frame = overridden_frame.parent.frame()
-            if isinstance(overridden_frame, astroid.ClassDef) and klass.is_subtype_of(
-                overridden_frame.qname()
+            if not (
+                isinstance(overridden_frame, astroid.ClassDef)
+                and klass.is_subtype_of(overridden_frame.qname())
             ):
-                args = (overridden.root().name, overridden.fromlineno)
-                self.add_message("method-hidden", args=args, node=node)
+                return
+
+            # If a subclass defined the method then it's not our fault.
+            for ancestor in klass.ancestors():
+                for obj in ancestor.lookup(node.name)[1]:
+                    if isinstance(obj, astroid.FunctionDef):
+                        return
+            args = (overridden.root().name, overridden.fromlineno)
+            self.add_message("method-hidden", args=args, node=node)
         except astroid.NotFoundError:
             pass
 
@@ -1019,7 +1071,7 @@ a metaclass class method.",
         ):
             return
 
-        #  Check values of default args
+        # Check values of default args
         klass = function.parent.frame()
         meth_node = None
         for overridden in klass.local_attr_ancestors(function.name):
@@ -1048,13 +1100,16 @@ a metaclass class method.",
 
         if meth_node is not None:
 
-            def form_annotations(annotations):
+            def form_annotations(arguments):
+                annotations = chain(
+                    (arguments.posonlyargs_annotations or []), arguments.annotations
+                )
                 return [
                     annotation.as_string() for annotation in filter(None, annotations)
                 ]
 
-            called_annotations = form_annotations(function.args.annotations)
-            overridden_annotations = form_annotations(meth_node.args.annotations)
+            called_annotations = form_annotations(function.args)
+            overridden_annotations = form_annotations(meth_node.args)
             if called_annotations and overridden_annotations:
                 if called_annotations != overridden_annotations:
                     return
@@ -1065,7 +1120,12 @@ a metaclass class method.",
             )
 
     def _check_property_with_parameters(self, node):
-        if node.args.args and len(node.args.args) > 1 and decorated_with_property(node):
+        if (
+            node.args.args
+            and len(node.args.args) > 1
+            and decorated_with_property(node)
+            and not is_property_setter(node)
+        ):
             self.add_message("property-with-parameters", node=node)
 
     def _check_invalid_overridden_method(self, function_node, parent_function_node):
@@ -1088,6 +1148,23 @@ a metaclass class method.",
                 node=function_node,
             )
 
+        parent_is_async = isinstance(parent_function_node, astroid.AsyncFunctionDef)
+        current_is_async = isinstance(function_node, astroid.AsyncFunctionDef)
+
+        if parent_is_async and not current_is_async:
+            self.add_message(
+                "invalid-overridden-method",
+                args=(function_node.name, "async", "non-async"),
+                node=function_node,
+            )
+
+        elif not parent_is_async and current_is_async:
+            self.add_message(
+                "invalid-overridden-method",
+                args=(function_node.name, "non-async", "async"),
+                node=function_node,
+            )
+
     def _check_slots(self, node):
         if "__slots__" not in node.locals:
             return
@@ -1169,6 +1246,7 @@ a metaclass class method.",
                     or decorated_with_property(node)
                     or _has_bare_super_call(node)
                     or is_protocol_class(class_node)
+                    or is_overload_stub(node)
                 )
             ):
                 self.add_message("no-self-use", node=node)
@@ -1196,7 +1274,7 @@ a metaclass class method.",
         self._check_in_slots(node)
 
     def _check_in_slots(self, node):
-        """ Check that the given AssignAttr node
+        """Check that the given AssignAttr node
         is defined in the class slots.
         """
         inferred = safe_infer(node.expr)
@@ -1349,11 +1427,12 @@ a metaclass class method.",
                     name = stmt.targets[0].name
                     if _is_attribute_property(name, klass):
                         return
-
-                #  A licit use of protected member is inside a special method
-                if not attrname.startswith(
-                    "__"
-                ) and self._is_called_inside_special_method(node):
+                licit_protected_member = not attrname.startswith("__")
+                if (
+                    not self.config.check_protected_access_in_special_methods
+                    and licit_protected_member
+                    and self._is_called_inside_special_method(node)
+                ):
                     return
 
                 self.add_message("protected-access", node=node, args=attrname)
@@ -1460,10 +1539,10 @@ a metaclass class method.",
         # don't care about functions with unknown argument (builtins)
         if node.args.args is None:
             return
-        if node.args.args:
-            first_arg = node.argnames()[0]
-        elif node.args.posonlyargs:
+        if node.args.posonlyargs:
             first_arg = node.args.posonlyargs[0].name
+        elif node.args.args:
+            first_arg = node.argnames()[0]
         else:
             first_arg = None
         self._first_attrs.append(first_arg)
@@ -1502,7 +1581,7 @@ a metaclass class method.",
                     node.name,
                 )
         # regular class
-        else:
+        else:  # pylint: disable=else-if-used
             # class method
             if node.type == "classmethod" or node.name == "__class_getitem__":
                 self._check_first_arg_config(
@@ -1522,7 +1601,7 @@ a metaclass class method.",
                 valid = repr(config[0])
             else:
                 valid = ", ".join(repr(v) for v in config[:-1])
-                valid = "%s or %r" % (valid, config[-1])
+                valid = f"{valid} or {config[-1]!r}"
             self.add_message(message, args=(method_name, valid), node=node)
 
     def _check_bases_classes(self, node):
@@ -1612,8 +1691,7 @@ a metaclass class method.",
             self.add_message("super-init-not-called", args=klass.name, node=node)
 
     def _check_signature(self, method1, refmethod, class_type, cls):
-        """check that the signature of the two given methods match
-        """
+        """check that the signature of the two given methods match"""
         if not (
             isinstance(method1, astroid.FunctionDef)
             and isinstance(refmethod, astroid.FunctionDef)
@@ -1645,7 +1723,10 @@ a metaclass class method.",
             self.add_message(
                 "arguments-differ", args=(class_type, method1.name), node=method1
             )
-        elif len(method1.args.defaults) < len(refmethod.args.defaults):
+        elif (
+            len(method1.args.defaults) < len(refmethod.args.defaults)
+            and not method1.args.vararg
+        ):
             self.add_message(
                 "signature-differs", args=(class_type, method1.name), node=method1
             )
@@ -1702,23 +1783,105 @@ class SpecialMethodsChecker(BaseChecker):
             "invalid-length-returned",
             "Used when a __len__ method returns something which is not a "
             "non-negative integer",
-            {},
+        ),
+        "E0304": (
+            "__bool__ does not return bool",
+            "invalid-bool-returned",
+            "Used when a __bool__ method returns something which is not a bool",
+        ),
+        "E0305": (
+            "__index__ does not return int",
+            "invalid-index-returned",
+            "Used when an __index__ method returns something which is not "
+            "an integer",
+        ),
+        "E0306": (
+            "__repr__ does not return str",
+            "invalid-repr-returned",
+            "Used when a __repr__ method returns something which is not a string",
+        ),
+        "E0307": (
+            "__str__ does not return str",
+            "invalid-str-returned",
+            "Used when a __str__ method returns something which is not a string",
+        ),
+        "E0308": (
+            "__bytes__ does not return bytes",
+            "invalid-bytes-returned",
+            "Used when a __bytes__ method returns something which is not bytes",
+        ),
+        "E0309": (
+            "__hash__ does not return int",
+            "invalid-hash-returned",
+            "Used when a __hash__ method returns something which is not an integer",
+        ),
+        "E0310": (
+            "__length_hint__ does not return non-negative integer",
+            "invalid-length-hint-returned",
+            "Used when a __length_hint__ method returns something which is not a "
+            "non-negative integer",
+        ),
+        "E0311": (
+            "__format__ does not return str",
+            "invalid-format-returned",
+            "Used when a __format__ method returns something which is not a string",
+        ),
+        "E0312": (
+            "__getnewargs__ does not return a tuple",
+            "invalid-getnewargs-returned",
+            "Used when a __getnewargs__ method returns something which is not "
+            "a tuple",
+        ),
+        "E0313": (
+            "__getnewargs_ex__ does not return a tuple containing (tuple, dict)",
+            "invalid-getnewargs-ex-returned",
+            "Used when a __getnewargs_ex__ method returns something which is not "
+            "of the form tuple(tuple, dict)",
         ),
     }
     priority = -2
 
+    def __init__(self, linter=None):
+        BaseChecker.__init__(self, linter)
+        self._protocol_map = {
+            "__iter__": self._check_iter,
+            "__len__": self._check_len,
+            "__bool__": self._check_bool,
+            "__index__": self._check_index,
+            "__repr__": self._check_repr,
+            "__str__": self._check_str,
+            "__bytes__": self._check_bytes,
+            "__hash__": self._check_hash,
+            "__length_hint__": self._check_length_hint,
+            "__format__": self._check_format,
+            "__getnewargs__": self._check_getnewargs,
+            "__getnewargs_ex__": self._check_getnewargs_ex,
+        }
+
     @check_messages(
         "unexpected-special-method-signature",
         "non-iterator-returned",
         "invalid-length-returned",
+        "invalid-bool-returned",
+        "invalid-index-returned",
+        "invalid-repr-returned",
+        "invalid-str-returned",
+        "invalid-bytes-returned",
+        "invalid-hash-returned",
+        "invalid-length-hint-returned",
+        "invalid-format-returned",
+        "invalid-getnewargs-returned",
+        "invalid-getnewargs-ex-returned",
     )
     def visit_functiondef(self, node):
         if not node.is_method():
             return
-        if node.name == "__iter__":
-            self._check_iter(node)
-        if node.name == "__len__":
-            self._check_len(node)
+
+        inferred = _safe_infer_call_result(node, node)
+        # Only want to check types that we are able to infer
+        if inferred and node.name in self._protocol_map:
+            self._protocol_map[node.name](node, inferred)
+
         if node.name in PYMETHODS:
             self._check_unexpected_method_signature(node)
 
@@ -1771,6 +1934,56 @@ class SpecialMethodsChecker(BaseChecker):
                 node=node,
             )
 
+    @staticmethod
+    def _is_wrapped_type(node, type_):
+        return (
+            isinstance(node, astroid.Instance)
+            and node.name == type_
+            and not isinstance(node, astroid.Const)
+        )
+
+    @staticmethod
+    def _is_int(node):
+        if SpecialMethodsChecker._is_wrapped_type(node, "int"):
+            return True
+
+        return isinstance(node, astroid.Const) and isinstance(node.value, int)
+
+    @staticmethod
+    def _is_str(node):
+        if SpecialMethodsChecker._is_wrapped_type(node, "str"):
+            return True
+
+        return isinstance(node, astroid.Const) and isinstance(node.value, str)
+
+    @staticmethod
+    def _is_bool(node):
+        if SpecialMethodsChecker._is_wrapped_type(node, "bool"):
+            return True
+
+        return isinstance(node, astroid.Const) and isinstance(node.value, bool)
+
+    @staticmethod
+    def _is_bytes(node):
+        if SpecialMethodsChecker._is_wrapped_type(node, "bytes"):
+            return True
+
+        return isinstance(node, astroid.Const) and isinstance(node.value, bytes)
+
+    @staticmethod
+    def _is_tuple(node):
+        if SpecialMethodsChecker._is_wrapped_type(node, "tuple"):
+            return True
+
+        return isinstance(node, astroid.Const) and isinstance(node.value, tuple)
+
+    @staticmethod
+    def _is_dict(node):
+        if SpecialMethodsChecker._is_wrapped_type(node, "dict"):
+            return True
+
+        return isinstance(node, astroid.Const) and isinstance(node.value, dict)
+
     @staticmethod
     def _is_iterator(node):
         if node is astroid.Uninferable:
@@ -1796,33 +2009,83 @@ class SpecialMethodsChecker(BaseChecker):
                     pass
         return False
 
-    def _check_iter(self, node):
-        inferred = _safe_infer_call_result(node, node)
-        if inferred is not None:
-            if not self._is_iterator(inferred):
-                self.add_message("non-iterator-returned", node=node)
+    def _check_iter(self, node, inferred):
+        if not self._is_iterator(inferred):
+            self.add_message("non-iterator-returned", node=node)
 
-    def _check_len(self, node):
-        inferred = _safe_infer_call_result(node, node)
-        if not inferred or inferred is astroid.Uninferable:
-            return
-
-        if (
-            isinstance(inferred, astroid.Instance)
-            and inferred.name == "int"
-            and not isinstance(inferred, astroid.Const)
-        ):
-            # Assume it's good enough, since the int() call might wrap
-            # something that's uninferable for us
-            return
-
-        if not isinstance(inferred, astroid.Const):
+    def _check_len(self, node, inferred):
+        if not self._is_int(inferred):
             self.add_message("invalid-length-returned", node=node)
+        elif isinstance(inferred, astroid.Const) and inferred.value < 0:
+            self.add_message("invalid-length-returned", node=node)
+
+    def _check_bool(self, node, inferred):
+        if not self._is_bool(inferred):
+            self.add_message("invalid-bool-returned", node=node)
+
+    def _check_index(self, node, inferred):
+        if not self._is_int(inferred):
+            self.add_message("invalid-index-returned", node=node)
+
+    def _check_repr(self, node, inferred):
+        if not self._is_str(inferred):
+            self.add_message("invalid-repr-returned", node=node)
+
+    def _check_str(self, node, inferred):
+        if not self._is_str(inferred):
+            self.add_message("invalid-str-returned", node=node)
+
+    def _check_bytes(self, node, inferred):
+        if not self._is_bytes(inferred):
+            self.add_message("invalid-bytes-returned", node=node)
+
+    def _check_hash(self, node, inferred):
+        if not self._is_int(inferred):
+            self.add_message("invalid-hash-returned", node=node)
+
+    def _check_length_hint(self, node, inferred):
+        if not self._is_int(inferred):
+            self.add_message("invalid-length-hint-returned", node=node)
+        elif isinstance(inferred, astroid.Const) and inferred.value < 0:
+            self.add_message("invalid-length-hint-returned", node=node)
+
+    def _check_format(self, node, inferred):
+        if not self._is_str(inferred):
+            self.add_message("invalid-format-returned", node=node)
+
+    def _check_getnewargs(self, node, inferred):
+        if not self._is_tuple(inferred):
+            self.add_message("invalid-getnewargs-returned", node=node)
+
+    def _check_getnewargs_ex(self, node, inferred):
+        if not self._is_tuple(inferred):
+            self.add_message("invalid-getnewargs-ex-returned", node=node)
             return
 
-        value = inferred.value
-        if not isinstance(value, int) or value < 0:
-            self.add_message("invalid-length-returned", node=node)
+        if not isinstance(inferred, astroid.Tuple):
+            # If it's not an astroid.Tuple we can't analyze it further
+            return
+
+        found_error = False
+
+        if len(inferred.elts) != 2:
+            found_error = True
+        else:
+            for arg, check in [
+                (inferred.elts[0], self._is_tuple),
+                (inferred.elts[1], self._is_dict),
+            ]:
+
+                if isinstance(arg, astroid.Call):
+                    arg = safe_infer(arg)
+
+                if arg and arg is not astroid.Uninferable:
+                    if not check(arg):
+                        found_error = True
+                        break
+
+        if found_error:
+            self.add_message("invalid-getnewargs-ex-returned", node=node)
 
 
 def _ancestors_to_call(klass_node, method="__init__"):
