@@ -1,8 +1,7 @@
-# -*- coding: utf-8 -*-
 # Copyright (c) 2006-2014 LOGILAB S.A. (Paris, FRANCE) <contact@logilab.fr>
 # Copyright (c) 2009 James Lingard <jchl@aristanetworks.com>
 # Copyright (c) 2012-2014 Google, Inc.
-# Copyright (c) 2014-2018 Claudiu Popa <pcmanticore@gmail.com>
+# Copyright (c) 2014-2020 Claudiu Popa <pcmanticore@gmail.com>
 # Copyright (c) 2014 David Shea <dshea@redhat.com>
 # Copyright (c) 2014 Steven Myint <hg@stevenmyint.com>
 # Copyright (c) 2014 Holger Peters <email@holger-peters.de>
@@ -12,21 +11,38 @@
 # Copyright (c) 2015 Rene Zhang <rz99@cornell.edu>
 # Copyright (c) 2015 Radu Ciorba <radu@devrandom.ro>
 # Copyright (c) 2015 Ionel Cristian Maries <contact@ionelmc.ro>
+# Copyright (c) 2016, 2019 Ashley Whetter <ashley@awhetter.co.uk>
 # Copyright (c) 2016 Alexander Todorov <atodorov@otb.bg>
-# Copyright (c) 2016 Ashley Whetter <ashley@awhetter.co.uk>
 # Copyright (c) 2016 Jürgen Hermann <jh@web.de>
 # Copyright (c) 2016 Jakub Wilk <jwilk@jwilk.net>
 # Copyright (c) 2016 Filipe Brandenburger <filbranden@google.com>
-# Copyright (c) 2017-2018 hippo91 <guillaume.peillex@gmail.com>
+# Copyright (c) 2017-2018, 2020 hippo91 <guillaume.peillex@gmail.com>
 # Copyright (c) 2017 Łukasz Rogalski <rogalski.91@gmail.com>
 # Copyright (c) 2017 Derek Gustafson <degustaf@gmail.com>
 # Copyright (c) 2017 Ville Skyttä <ville.skytta@iki.fi>
-# Copyright (c) 2018 Nick Drozd <nicholasdrozd@gmail.com>
+# Copyright (c) 2018-2019 Nick Drozd <nicholasdrozd@gmail.com>
+# Copyright (c) 2018 Pablo Galindo <Pablogsal@gmail.com>
+# Copyright (c) 2018 Jim Robertson <jrobertson98atx@gmail.com>
+# Copyright (c) 2018 Lucas Cimon <lucas.cimon@gmail.com>
 # Copyright (c) 2018 Mike Frysinger <vapier@gmail.com>
 # Copyright (c) 2018 Ben Green <benhgreen@icloud.com>
 # Copyright (c) 2018 Konstantin <Github@pheanex.de>
 # Copyright (c) 2018 Justin Li <justinnhli@users.noreply.github.com>
 # Copyright (c) 2018 Bryce Guinta <bryce.paul.guinta@gmail.com>
+# Copyright (c) 2019-2021 Pierre Sassoulas <pierre.sassoulas@gmail.com>
+# Copyright (c) 2019 Andy Palmer <25123779+ninezerozeronine@users.noreply.github.com>
+# Copyright (c) 2019 mattlbeck <17108752+mattlbeck@users.noreply.github.com>
+# Copyright (c) 2019 Martin Vielsmaier <martin.vielsmaier@gmail.com>
+# Copyright (c) 2019 Santiago Castro <bryant@montevideo.com.uy>
+# Copyright (c) 2019 yory8 <39745367+yory8@users.noreply.github.com>
+# Copyright (c) 2019 Federico Bond <federicobond@gmail.com>
+# Copyright (c) 2019 Pascal Corpet <pcorpet@users.noreply.github.com>
+# Copyright (c) 2020 Julien Palard <julien@palard.fr>
+# Copyright (c) 2020 Ram Rachum <ram@rachum.com>
+# Copyright (c) 2020 Anthony Sottile <asottile@umich.edu>
+# Copyright (c) 2020 Anubhav <35621759+anubh-v@users.noreply.github.com>
+# Copyright (c) 2021 Ikraduya Edian <ikraduya@gmail.com>
+# Copyright (c) 2021 Marc Mueller <30130371+cdce8p@users.noreply.github.com>
 
 # Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
 # For details: https://github.com/PyCQA/pylint/blob/master/COPYING
@@ -46,26 +62,29 @@ import types
 from collections import deque
 from collections.abc import Sequence
 from functools import singledispatch
+from typing import Pattern, Tuple
 
 import astroid
 import astroid.arguments
 import astroid.context
 import astroid.nodes
-from astroid import bases, decorators, exceptions, modutils, objects
+from astroid import bases, decorators, exceptions, helpers, modutils, objects
 from astroid.interpreter import dunder_lookup
 
-from pylint.checkers import BaseChecker
+from pylint.checkers import BaseChecker, utils
 from pylint.checkers.utils import (
     check_messages,
     decorated_with,
     decorated_with_property,
     has_known_bases,
     is_builtin_object,
+    is_classdef_type,
     is_comprehension,
     is_inside_abstract_class,
     is_iterable,
     is_mapping,
     is_overload_stub,
+    is_postponed_evaluation_enabled,
     is_super,
     node_ignores_exception,
     safe_infer,
@@ -74,19 +93,20 @@ from pylint.checkers.utils import (
     supports_membership_test,
     supports_setitem,
 )
+from pylint.constants import PY310_PLUS
 from pylint.interfaces import INFERENCE, IAstroidChecker
 from pylint.utils import get_global_option
 
 BUILTINS = builtins.__name__
 STR_FORMAT = {"%s.str.format" % BUILTINS}
 ASYNCIO_COROUTINE = "asyncio.coroutines.coroutine"
+BUILTIN_TUPLE = "builtins.tuple"
 
 
 def _unflatten(iterable):
     for index, elem in enumerate(iterable):
         if isinstance(elem, Sequence) and not isinstance(elem, str):
-            for single_elem in _unflatten(elem):
-                yield single_elem
+            yield from _unflatten(elem)
         elif elem and not index:
             # We're interested only in the first element.
             yield elem
@@ -136,7 +156,7 @@ def _is_owner_ignored(owner, attrname, ignored_classes, ignored_modules):
             if not current_module:
                 current_module = part
             else:
-                current_module += ".{}".format(part)
+                current_module += f".{part}"
             if current_module in ignored_modules:
                 return True
 
@@ -228,7 +248,7 @@ def _missing_member_hint(owner, attrname, distance_threshold, max_choices):
     else:
         names = "one of {} or {}".format(", ".join(names[:-1]), names[-1])
 
-    return "; maybe {}?".format(names)
+    return f"; maybe {names}?"
 
 
 MSGS = {
@@ -389,6 +409,16 @@ MSGS = {
         "Emitted  when the caller's argument names fully match the parameter "
         "names in the function signature but do not have the same order.",
     ),
+    "W1115": (
+        "Non-string value assigned to __name__",
+        "non-str-assignment-to-dunder-name",
+        "Emitted when a non-string vaue is assigned to __name__",
+    ),
+    "W1116": (
+        "Second argument of isinstance is not a type",
+        "isinstance-second-argument-not-valid-type",
+        "Emitted when the second argument of an isinstance call is not a type.",
+    ),
 }
 
 # builtin sequence types in Python 2 and 3.
@@ -476,6 +506,18 @@ def _emit_no_member(node, owner, owner_name, ignored_mixins=True, ignored_none=T
                 return False
         except astroid.NotFoundError:
             return True
+    if (
+        owner.parent
+        and isinstance(owner.parent, astroid.ClassDef)
+        and owner.parent.name == "EnumMeta"
+        and owner_name == "__members__"
+        and node.attrname in ["items", "values", "keys"]
+    ):
+        print(node.attrname)
+        # Avoid false positive on Enum.__members__.{items(), values, keys}
+        # See https://github.com/PyCQA/pylint/issues/4123
+        return False
+
     return True
 
 
@@ -513,9 +555,9 @@ def _determine_callable(callable_obj):
             try:
                 # Use the last definition of __init__.
                 callable_obj = callable_obj.local_attr("__init__")[-1]
-            except exceptions.NotFoundError:
+            except exceptions.NotFoundError as e:
                 # do nothing, covered by no-init.
-                raise ValueError
+                raise ValueError from e
         else:
             callable_obj = new
 
@@ -541,12 +583,11 @@ def _no_context_variadic_keywords(node, scope):
 
     if isinstance(scope, astroid.Lambda) and not isinstance(scope, astroid.FunctionDef):
         variadics = list(node.keywords or []) + node.kwargs
-    else:
-        if isinstance(statement, (astroid.Return, astroid.Expr)) and isinstance(
-            statement.value, astroid.Call
-        ):
-            call = statement.value
-            variadics = list(call.keywords or []) + call.kwargs
+    elif isinstance(
+        statement, (astroid.Return, astroid.Expr, astroid.Assign)
+    ) and isinstance(statement.value, astroid.Call):
+        call = statement.value
+        variadics = list(call.keywords or []) + call.kwargs
 
     return _no_context_variadic(node, scope.args.kwarg, astroid.Keyword, variadics)
 
@@ -557,9 +598,9 @@ def _no_context_variadic_positional(node, scope):
         variadics = node.starargs + node.kwargs
     else:
         statement = node.statement()
-        if isinstance(statement, (astroid.Expr, astroid.Return)) and isinstance(
-            statement.value, astroid.Call
-        ):
+        if isinstance(
+            statement, (astroid.Expr, astroid.Return, astroid.Assign)
+        ) and isinstance(statement.value, astroid.Call):
             call = statement.value
             variadics = call.starargs + call.kwargs
 
@@ -668,9 +709,23 @@ def _is_c_extension(module_node):
     )
 
 
+def _is_invalid_isinstance_type(arg):
+    # Return True if we are sure that arg is not a type
+    inferred = utils.safe_infer(arg)
+    if not inferred:
+        # Cannot infer it so skip it.
+        return False
+    if isinstance(inferred, astroid.Tuple):
+        return any(_is_invalid_isinstance_type(elt) for elt in inferred.elts)
+    if isinstance(inferred, astroid.ClassDef):
+        return False
+    if isinstance(inferred, astroid.Instance) and inferred.qname() == BUILTIN_TUPLE:
+        return False
+    return True
+
+
 class TypeChecker(BaseChecker):
-    """try to find bugs in the code using type inference
-    """
+    """try to find bugs in the code using type inference"""
 
     __implements__ = (IAstroidChecker,)
 
@@ -815,17 +870,20 @@ accessed. Python regular expressions are accepted.",
     def _suggestion_mode(self):
         return get_global_option(self, "suggestion-mode", default=True)
 
-    def open(self):
-        # do this in open since config not fully initialized in __init__
+    @decorators.cachedproperty
+    def _compiled_generated_members(self) -> Tuple[Pattern, ...]:
+        # do this lazily since config not fully initialized in __init__
         # generated_members may contain regular expressions
         # (surrounded by quote `"` and followed by a comma `,`)
         # REQUEST,aq_parent,"[a-zA-Z]+_set{1,2}"' =>
         # ('REQUEST', 'aq_parent', '[a-zA-Z]+_set{1,2}')
-        if isinstance(self.config.generated_members, str):
-            gen = shlex.shlex(self.config.generated_members)
+        generated_members = self.config.generated_members
+        if isinstance(generated_members, str):
+            gen = shlex.shlex(generated_members)
             gen.whitespace += ","
             gen.wordchars += r"[]-+\.*?()|"
-            self.config.generated_members = tuple(tok.strip('"') for tok in gen)
+            generated_members = tuple(tok.strip('"') for tok in gen)
+        return tuple(re.compile(exp) for exp in generated_members)
 
     @check_messages("keyword-arg-before-vararg")
     def visit_functiondef(self, node):
@@ -879,12 +937,12 @@ accessed. Python regular expressions are accepted.",
 
         function/method, super call and metaclasses are ignored
         """
-        for pattern in self.config.generated_members:
-            # attribute is marked as generated, stop here
-            if re.match(pattern, node.attrname):
-                return
-            if re.match(pattern, node.as_string()):
-                return
+        if any(
+            pattern.match(name)
+            for name in (node.attrname, node.as_string())
+            for pattern in self._compiled_generated_members
+        ):
+            return
 
         try:
             inferred = list(node.expr.infer())
@@ -915,6 +973,12 @@ accessed. Python regular expressions are accepted.",
             ):
                 continue
 
+            qualname = f"{owner.pytype()}.{node.attrname}"
+            if any(
+                pattern.match(qualname) for pattern in self._compiled_generated_members
+            ):
+                return
+
             try:
                 if not [
                     n
@@ -986,8 +1050,20 @@ accessed. Python regular expressions are accepted.",
                 hint = ""
         return msg, hint
 
-    @check_messages("assignment-from-no-return", "assignment-from-none")
+    @check_messages(
+        "assignment-from-no-return",
+        "assignment-from-none",
+        "non-str-assignment-to-dunder-name",
+    )
     def visit_assign(self, node):
+        """
+        Process assignments in the AST.
+        """
+
+        self._check_assignment_from_function_call(node)
+        self._check_dundername_is_string(node)
+
+    def _check_assignment_from_function_call(self, node):
         """check that if assigning to a function call, the function is
         possibly returning something valuable
         """
@@ -1014,6 +1090,7 @@ accessed. Python regular expressions are accepted.",
             or function_node.decorators
             or function_node.is_generator()
             or function_node.is_abstract(pass_is_abstract=False)
+            or utils.is_error(function_node)
             or not function_node.root().fully_defined()
         ):
             return
@@ -1034,6 +1111,31 @@ accessed. Python regular expressions are accepted.",
             else:
                 self.add_message("assignment-from-none", node=node)
 
+    def _check_dundername_is_string(self, node):
+        """
+        Check a string is assigned to self.__name__
+        """
+
+        # Check the left hand side of the assignment is <something>.__name__
+        lhs = node.targets[0]
+        if not isinstance(lhs, astroid.node_classes.AssignAttr):
+            return
+        if not lhs.attrname == "__name__":
+            return
+
+        # If the right hand side is not a string
+        rhs = node.value
+        if isinstance(rhs, astroid.Const) and isinstance(rhs.value, str):
+            return
+        inferred = utils.safe_infer(rhs)
+        if not inferred:
+            return
+        if not (
+            isinstance(inferred, astroid.Const) and isinstance(inferred.value, str)
+        ):
+            # Add the message
+            self.add_message("non-str-assignment-to-dunder-name", node=node)
+
     def _check_uninferable_call(self, node):
         """
         Check that the given uninferable Call node does not
@@ -1124,6 +1226,15 @@ accessed. Python regular expressions are accepted.",
         if calling_parg_names != called_param_names[: len(calling_parg_names)]:
             self.add_message("arguments-out-of-order", node=node, args=())
 
+    def _check_isinstance_args(self, node):
+        if len(node.args) != 2:
+            # isinstance called with wrong number of args
+            return
+
+        second_arg = node.args[1]
+        if _is_invalid_isinstance_type(second_arg):
+            self.add_message("isinstance-second-argument-not-valid-type", node=node)
+
     # pylint: disable=too-many-branches,too-many-locals
     @check_messages(*(list(MSGS.keys())))
     def visit_call(self, node):
@@ -1158,6 +1269,9 @@ accessed. Python regular expressions are accepted.",
             return
 
         if called.args.args is None:
+            if called.name == "isinstance":
+                # Verify whether second argument of isinstance is a valid type
+                self._check_isinstance_args(node)
             # Built-in functions have no argument information.
             return
 
@@ -1252,13 +1366,12 @@ accessed. Python regular expressions are accepted.",
                 # The remaining positional arguments get assigned to the *args
                 # parameter.
                 break
-            else:
-                if not overload_function:
-                    # Too many positional arguments.
-                    self.add_message(
-                        "too-many-function-args", node=node, args=(callable_name,)
-                    )
-                    break
+            elif not overload_function:
+                # Too many positional arguments.
+                self.add_message(
+                    "too-many-function-args", node=node, args=(callable_name,)
+                )
+                break
 
         # 2. Match the keyword arguments.
         for keyword in keyword_args:
@@ -1327,23 +1440,19 @@ accessed. Python regular expressions are accepted.",
 
         for name in kwparams:
             defval, assigned = kwparams[name]
-            if defval is None and not assigned and not has_no_context_keywords_variadic:
+            if (
+                defval is None
+                and not assigned
+                and not has_no_context_keywords_variadic
+                and not overload_function
+            ):
                 self.add_message("missing-kwoa", node=node, args=(name, callable_name))
 
-    @check_messages("invalid-sequence-index")
-    def visit_extslice(self, node):
-        # Check extended slice objects as if they were used as a sequence
-        # index to check if the object being sliced can support them
-        return self.visit_index(node)
-
-    @check_messages("invalid-sequence-index")
-    def visit_index(self, node):
-        if not node.parent or not hasattr(node.parent, "value"):
-            return None
+    def _check_invalid_sequence_index(self, subscript: astroid.Subscript):
         # Look for index operations where the parent is a sequence type.
         # If the types can be determined, only allow indices to be int,
         # slice or instances with __index__.
-        parent_type = safe_infer(node.parent.value)
+        parent_type = safe_infer(subscript.value)
         if not isinstance(
             parent_type, (astroid.ClassDef, astroid.Instance)
         ) or not has_known_bases(parent_type):
@@ -1352,9 +1461,9 @@ accessed. Python regular expressions are accepted.",
         # Determine what method on the parent this index will use
         # The parent of this node will be a Subscript, and the parent of that
         # node determines if the Subscript is a get, set, or delete operation.
-        if node.parent.ctx is astroid.Store:
+        if subscript.ctx is astroid.Store:
             methodname = "__setitem__"
-        elif node.parent.ctx is astroid.Del:
+        elif subscript.ctx is astroid.Del:
             methodname = "__delitem__"
         else:
             methodname = "__getitem__"
@@ -1386,10 +1495,10 @@ accessed. Python regular expressions are accepted.",
         # For ExtSlice objects coming from visit_extslice, no further
         # inference is necessary, since if we got this far the ExtSlice
         # is an error.
-        if isinstance(node, astroid.ExtSlice):
-            index_type = node
+        if isinstance(subscript.value, astroid.ExtSlice):
+            index_type = subscript.value
         else:
-            index_type = safe_infer(node)
+            index_type = safe_infer(subscript.slice)
         if index_type is None or index_type is astroid.Uninferable:
             return None
         # Constants must be of type int
@@ -1406,17 +1515,24 @@ accessed. Python regular expressions are accepted.",
             except exceptions.NotFoundError:
                 pass
         elif isinstance(index_type, astroid.Slice):
-            # Delegate to visit_slice. A slice can be present
+            # A slice can be present
             # here after inferring the index node, which could
             # be a `slice(...)` call for instance.
-            return self.visit_slice(index_type)
+            return self._check_invalid_slice_index(index_type)
 
         # Anything else is an error
-        self.add_message("invalid-sequence-index", node=node)
+        self.add_message("invalid-sequence-index", node=subscript)
         return None
 
-    @check_messages("invalid-slice-index")
-    def visit_slice(self, node):
+    @check_messages("invalid-sequence-index")
+    def visit_extslice(self, node):
+        if not node.parent or not hasattr(node.parent, "value"):
+            return None
+        # Check extended slice objects as if they were used as a sequence
+        # index to check if the object being sliced can support them
+        return self._check_invalid_sequence_index(node.parent)
+
+    def _check_invalid_slice_index(self, node):
         # Check the type of each part of the slice
         invalid_slices = 0
         for index in (node.lower, node.upper, node.step):
@@ -1540,6 +1656,53 @@ accessed. Python regular expressions are accepted.",
             # Let the error customize its output.
             self.add_message("invalid-unary-operand-type", args=str(error), node=node)
 
+    @check_messages("unsupported-binary-operation")
+    def visit_binop(self, node: astroid.BinOp):
+        # Test alternative Union syntax PEP 604 - int | None
+        msg = "unsupported operand type(s) for |"
+        if node.op == "|" and (
+            not is_postponed_evaluation_enabled(node)
+            and isinstance(
+                node.parent, (astroid.AnnAssign, astroid.Arguments, astroid.FunctionDef)
+            )
+            or not PY310_PLUS
+            and isinstance(
+                node.parent,
+                (
+                    astroid.Assign,
+                    astroid.Call,
+                    astroid.Keyword,
+                    astroid.Dict,
+                    astroid.Tuple,
+                    astroid.Set,
+                    astroid.List,
+                    astroid.BinOp,
+                ),
+            )
+        ):
+            allowed_nested_syntax = False
+            if is_postponed_evaluation_enabled(node):
+                parent_node = node.parent
+                while True:
+                    if isinstance(
+                        parent_node,
+                        (astroid.AnnAssign, astroid.Arguments, astroid.FunctionDef),
+                    ):
+                        allowed_nested_syntax = True
+                        break
+                    parent_node = parent_node.parent
+                    if isinstance(parent_node, astroid.Module):
+                        break
+            if allowed_nested_syntax is False:
+                for n in (node.left, node.right):
+                    n = helpers.object_type(n)
+                    if isinstance(n, astroid.ClassDef):
+                        if is_classdef_type(n):
+                            self.add_message(
+                                "unsupported-binary-operation", args=msg, node=node
+                            )
+                            break
+
     @check_messages("unsupported-binary-operation")
     def _visit_binop(self, node):
         """Detect TypeErrors for binary arithmetic operands."""
@@ -1587,15 +1750,23 @@ accessed. Python regular expressions are accepted.",
         "unsupported-assignment-operation",
         "unsupported-delete-operation",
         "unhashable-dict-key",
+        "invalid-sequence-index",
+        "invalid-slice-index",
     )
     def visit_subscript(self, node):
+        self._check_invalid_sequence_index(node)
+
         supported_protocol = None
         if isinstance(node.value, (astroid.ListComp, astroid.DictComp)):
             return
 
         if isinstance(node.value, astroid.Dict):
             # Assert dict key is hashable
-            inferred = safe_infer(node.slice.value)
+            if isinstance(node.slice, (astroid.Index, astroid.Slice)):
+                # In Python 3.9 the Index, Slice and ExtSlice nodes are no longer in use
+                inferred = safe_infer(node.slice.value)
+            else:
+                inferred = safe_infer(node.slice)
             if inferred not in (None, astroid.Uninferable):
                 try:
                     hash_fn = next(inferred.igetattr("__hash__"))
@@ -1623,10 +1794,19 @@ accessed. Python regular expressions are accepted.",
             return
 
         inferred = safe_infer(node.value)
+
         if inferred is None or inferred is astroid.Uninferable:
             return
 
-        if not supported_protocol(inferred):
+        if getattr(inferred, "decorators", None):
+            first_decorator = helpers.safe_infer(inferred.decorators.nodes[0])
+            if isinstance(first_decorator, astroid.ClassDef):
+                inferred = first_decorator.instantiate_class()
+            else:
+                return  # It would be better to handle function
+                # decorators, but let's start slow.
+
+        if not supported_protocol(inferred, node):
             self.add_message(msg, args=node.value.as_string(), node=node.value)
 
     @check_messages("dict-items-missing-iter")
