@@ -4,10 +4,10 @@
 class NotUselessSuper(object):
 
     def not_passing_keyword_only(self, first, *, second):
-        return super(NotUselessSuper, self).not_passing_keyword_only(first)
+        return super().not_passing_keyword_only(first)
 
     def passing_keyword_only_with_modifications(self, first, *, second):
-        return super(NotUselessSuper, self).passing_keyword_only_with_modifications(
+        return super().passing_keyword_only_with_modifications(
             first, second + 1)
 
 
@@ -19,7 +19,7 @@ class AlsoNotUselessSuper(NotUselessSuper):
 class UselessSuper(object):
 
     def useless(self, *, first): # [useless-super-delegation]
-        super(UselessSuper, self).useless(first=first)
+        super().useless(first=first)
 
 
 class Egg():
